/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2021 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */
/* USER CODE END Header */
/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include "ssd1306.h"
#include "fonts.h"
#include <stdio.h>
#include <stdlib.h>
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
ADC_HandleTypeDef hadc;

I2C_HandleTypeDef hi2c1;
DMA_HandleTypeDef hdma_i2c1_tx;

TIM_HandleTypeDef htim3;
TIM_HandleTypeDef htim6;
TIM_HandleTypeDef htim14;

UART_HandleTypeDef huart1;
DMA_HandleTypeDef hdma_usart1_rx;

/* USER CODE BEGIN PV */
int lastFuelGaugeCount = 5;
int tstCnt = 0;
int cnt = 1;
int resetTimer = 0;
uint32_t lastRpmPulseTimCnt = 0;
uint32_t lastSpdPulseTimCnt = 0;
int rpm, speed;
float trip = 0;
float all_kms = 0;
float battVoltage = 0;
float fuelLevel = 0;
int battVoltageADC = 0;
int fuelLevelADC = 0;
uint32_t avgRpmTime = 0;
uint32_t avgSpdTime = 0;
uint32_t rpmSamples[60];
int rpmSamplesIndex = 0;
uint32_t spdSamples[60];
int spdSamplesIndex = 0;
/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_DMA_Init(void);
static void MX_I2C1_Init(void);
static void MX_ADC_Init(void);
static void MX_TIM3_Init(void);
static void MX_TIM14_Init(void);
static void MX_USART1_UART_Init(void);
static void MX_TIM6_Init(void);
/* USER CODE BEGIN PFP */
void FuelGauge(int);
void MultiChannelADC(ADC_HandleTypeDef*, uint32_t*, uint8_t);
/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_DMA_Init();
  MX_I2C1_Init();
  MX_ADC_Init();
  MX_TIM3_Init();
  MX_TIM14_Init();
  MX_USART1_UART_Init();
  MX_TIM6_Init();
  /* USER CODE BEGIN 2 */
  TIM6->DIER |= 1 << 0;

  if (ssd1306_Init(&hi2c1) != 0) {
    Error_Handler();
  }

  ssd1306_Fill(Black);
  ssd1306_UpdateScreen(&hi2c1);

  ssd1306_SetCursor(0, 23);
  ssd1306_WriteString("Motorcycle Monitor", Font_7x10, White);
  ssd1306_SetCursor(23, 45);
  ssd1306_WriteString("Ver: 1.0.0", Font_7x10, White);
  ssd1306_UpdateScreen(&hi2c1);
  HAL_Delay(1000);
  HAL_TIM_Base_Start_IT(&htim6);
  ssd1306_Fill(Black);

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {
	  uint32_t ADCReadArr[2];

	  MultiChannelADC(&hadc, ADCReadArr, 2);

	  fuelLevelADC = ADCReadArr[0];
	  battVoltageADC = ADCReadArr[1];

	  battVoltage = battVoltageADC*(V_REF/4096)*U_BATT_DIV_RATIO;
	  fuelLevel = fuelLevelADC*(V_REF/4096)*FUEL_DIV_RATIO;

	  int fuelLevelStep = MAX_FUEL_U/5;
	  if(fuelLevel < fuelLevelStep){
		  FuelGauge(0);
	  }
	  else if(fuelLevel < fuelLevelStep*2){
		  FuelGauge(1);
	  }
	  else if(fuelLevel < fuelLevelStep*3){
		  FuelGauge(2);
	  }
	  else if(fuelLevel < fuelLevelStep*4){
		  FuelGauge(3);
	  }
	  else{
		  FuelGauge(4);
	  }

	  if(resetTimer > 5){
		  resetTimer = 0;
		  ssd1306_SetCursor(0, 0);
		  ssd1306_WriteString("0", Font_16x26, White);
		  ssd1306_SetCursor(78, 0);
		  ssd1306_WriteString("0", Font_16x26, White);
		  ssd1306_UpdateScreen(&hi2c1);
	  }

    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};
  RCC_PeriphCLKInitTypeDef PeriphClkInit = {0};

  /** Initializes the RCC Oscillators according to the specified parameters
  * in the RCC_OscInitTypeDef structure.
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI|RCC_OSCILLATORTYPE_HSI14;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSI14State = RCC_HSI14_ON;
  RCC_OscInitStruct.HSICalibrationValue = RCC_HSICALIBRATION_DEFAULT;
  RCC_OscInitStruct.HSI14CalibrationValue = 16;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSI;
  RCC_OscInitStruct.PLL.PLLMUL = RCC_PLL_MUL12;
  RCC_OscInitStruct.PLL.PREDIV = RCC_PREDIV_DIV1;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
  /** Initializes the CPU, AHB and APB buses clocks
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_1) != HAL_OK)
  {
    Error_Handler();
  }
  PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_USART1|RCC_PERIPHCLK_I2C1;
  PeriphClkInit.Usart1ClockSelection = RCC_USART1CLKSOURCE_PCLK1;
  PeriphClkInit.I2c1ClockSelection = RCC_I2C1CLKSOURCE_HSI;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK)
  {
    Error_Handler();
  }
  HAL_RCC_MCOConfig(RCC_MCO, RCC_MCO1SOURCE_SYSCLK, RCC_MCODIV_1);
}

/**
  * @brief ADC Initialization Function
  * @param None
  * @retval None
  */
static void MX_ADC_Init(void)
{

  /* USER CODE BEGIN ADC_Init 0 */

  /* USER CODE END ADC_Init 0 */

  ADC_ChannelConfTypeDef sConfig = {0};

  /* USER CODE BEGIN ADC_Init 1 */

  /* USER CODE END ADC_Init 1 */
  /** Configure the global features of the ADC (Clock, Resolution, Data Alignment and number of conversion)
  */
  hadc.Instance = ADC1;
  hadc.Init.ClockPrescaler = ADC_CLOCK_ASYNC_DIV1;
  hadc.Init.Resolution = ADC_RESOLUTION_12B;
  hadc.Init.DataAlign = ADC_DATAALIGN_RIGHT;
  hadc.Init.ScanConvMode = ADC_SCAN_DIRECTION_FORWARD;
  hadc.Init.EOCSelection = ADC_EOC_SINGLE_CONV;
  hadc.Init.LowPowerAutoWait = DISABLE;
  hadc.Init.LowPowerAutoPowerOff = DISABLE;
  hadc.Init.ContinuousConvMode = DISABLE;
  hadc.Init.DiscontinuousConvMode = DISABLE;
  hadc.Init.ExternalTrigConv = ADC_SOFTWARE_START;
  hadc.Init.ExternalTrigConvEdge = ADC_EXTERNALTRIGCONVEDGE_NONE;
  hadc.Init.DMAContinuousRequests = DISABLE;
  hadc.Init.Overrun = ADC_OVR_DATA_PRESERVED;
  if (HAL_ADC_Init(&hadc) != HAL_OK)
  {
    Error_Handler();
  }
  /** Configure for the selected ADC regular channel to be converted.
  */
  sConfig.Channel = ADC_CHANNEL_3;
  sConfig.Rank = ADC_RANK_CHANNEL_NUMBER;
  sConfig.SamplingTime = ADC_SAMPLETIME_239CYCLES_5;
  if (HAL_ADC_ConfigChannel(&hadc, &sConfig) != HAL_OK)
  {
    Error_Handler();
  }
  /** Configure for the selected ADC regular channel to be converted.
  */
  sConfig.Channel = ADC_CHANNEL_5;
  if (HAL_ADC_ConfigChannel(&hadc, &sConfig) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN ADC_Init 2 */

  /* USER CODE END ADC_Init 2 */

}

/**
  * @brief I2C1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_I2C1_Init(void)
{

  /* USER CODE BEGIN I2C1_Init 0 */

  /* USER CODE END I2C1_Init 0 */

  /* USER CODE BEGIN I2C1_Init 1 */

  /* USER CODE END I2C1_Init 1 */
  hi2c1.Instance = I2C1;
  hi2c1.Init.Timing = 0x0000020B;
  hi2c1.Init.OwnAddress1 = 0;
  hi2c1.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  hi2c1.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  hi2c1.Init.OwnAddress2 = 0;
  hi2c1.Init.OwnAddress2Masks = I2C_OA2_NOMASK;
  hi2c1.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  hi2c1.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  if (HAL_I2C_Init(&hi2c1) != HAL_OK)
  {
    Error_Handler();
  }
  /** Configure Analogue filter
  */
  if (HAL_I2CEx_ConfigAnalogFilter(&hi2c1, I2C_ANALOGFILTER_ENABLE) != HAL_OK)
  {
    Error_Handler();
  }
  /** Configure Digital filter
  */
  if (HAL_I2CEx_ConfigDigitalFilter(&hi2c1, 0) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN I2C1_Init 2 */

  /* USER CODE END I2C1_Init 2 */

}

/**
  * @brief TIM3 Initialization Function
  * @param None
  * @retval None
  */
static void MX_TIM3_Init(void)
{

  /* USER CODE BEGIN TIM3_Init 0 */

  /* USER CODE END TIM3_Init 0 */

  TIM_ClockConfigTypeDef sClockSourceConfig = {0};
  TIM_MasterConfigTypeDef sMasterConfig = {0};
  TIM_IC_InitTypeDef sConfigIC = {0};

  /* USER CODE BEGIN TIM3_Init 1 */

  /* USER CODE END TIM3_Init 1 */
  htim3.Instance = TIM3;
  htim3.Init.Prescaler = 4799;
  htim3.Init.CounterMode = TIM_COUNTERMODE_UP;
  htim3.Init.Period = 10000;
  htim3.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
  htim3.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_DISABLE;
  if (HAL_TIM_Base_Init(&htim3) != HAL_OK)
  {
    Error_Handler();
  }
  sClockSourceConfig.ClockSource = TIM_CLOCKSOURCE_INTERNAL;
  if (HAL_TIM_ConfigClockSource(&htim3, &sClockSourceConfig) != HAL_OK)
  {
    Error_Handler();
  }
  if (HAL_TIM_IC_Init(&htim3) != HAL_OK)
  {
    Error_Handler();
  }
  sMasterConfig.MasterOutputTrigger = TIM_TRGO_RESET;
  sMasterConfig.MasterSlaveMode = TIM_MASTERSLAVEMODE_DISABLE;
  if (HAL_TIMEx_MasterConfigSynchronization(&htim3, &sMasterConfig) != HAL_OK)
  {
    Error_Handler();
  }
  sConfigIC.ICPolarity = TIM_INPUTCHANNELPOLARITY_RISING;
  sConfigIC.ICSelection = TIM_ICSELECTION_DIRECTTI;
  sConfigIC.ICPrescaler = TIM_ICPSC_DIV1;
  sConfigIC.ICFilter = 0;
  if (HAL_TIM_IC_ConfigChannel(&htim3, &sConfigIC, TIM_CHANNEL_1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN TIM3_Init 2 */

  /* USER CODE END TIM3_Init 2 */

}

/**
  * @brief TIM6 Initialization Function
  * @param None
  * @retval None
  */
static void MX_TIM6_Init(void)
{

  /* USER CODE BEGIN TIM6_Init 0 */

  /* USER CODE END TIM6_Init 0 */

  /* USER CODE BEGIN TIM6_Init 1 */

  /* USER CODE END TIM6_Init 1 */
  htim6.Instance = TIM6;
  htim6.Init.Prescaler = 47999;
  htim6.Init.CounterMode = TIM_COUNTERMODE_UP;
  htim6.Init.Period = 1000;
  htim6.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_DISABLE;
  if (HAL_TIM_Base_Init(&htim6) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN TIM6_Init 2 */

  /* USER CODE END TIM6_Init 2 */

}

/**
  * @brief TIM14 Initialization Function
  * @param None
  * @retval None
  */
static void MX_TIM14_Init(void)
{

  /* USER CODE BEGIN TIM14_Init 0 */

  /* USER CODE END TIM14_Init 0 */

  TIM_IC_InitTypeDef sConfigIC = {0};

  /* USER CODE BEGIN TIM14_Init 1 */

  /* USER CODE END TIM14_Init 1 */
  htim14.Instance = TIM14;
  htim14.Init.Prescaler = 4799;
  htim14.Init.CounterMode = TIM_COUNTERMODE_UP;
  htim14.Init.Period = 10000;
  htim14.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
  htim14.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_DISABLE;
  if (HAL_TIM_Base_Init(&htim14) != HAL_OK)
  {
    Error_Handler();
  }
  if (HAL_TIM_IC_Init(&htim14) != HAL_OK)
  {
    Error_Handler();
  }
  sConfigIC.ICPolarity = TIM_INPUTCHANNELPOLARITY_RISING;
  sConfigIC.ICSelection = TIM_ICSELECTION_DIRECTTI;
  sConfigIC.ICPrescaler = TIM_ICPSC_DIV1;
  sConfigIC.ICFilter = 0;
  if (HAL_TIM_IC_ConfigChannel(&htim14, &sConfigIC, TIM_CHANNEL_1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN TIM14_Init 2 */

  /* USER CODE END TIM14_Init 2 */

}

/**
  * @brief USART1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_USART1_UART_Init(void)
{

  /* USER CODE BEGIN USART1_Init 0 */

  /* USER CODE END USART1_Init 0 */

  /* USER CODE BEGIN USART1_Init 1 */

  /* USER CODE END USART1_Init 1 */
  huart1.Instance = USART1;
  huart1.Init.BaudRate = 38400;
  huart1.Init.WordLength = UART_WORDLENGTH_8B;
  huart1.Init.StopBits = UART_STOPBITS_1;
  huart1.Init.Parity = UART_PARITY_NONE;
  huart1.Init.Mode = UART_MODE_TX_RX;
  huart1.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart1.Init.OverSampling = UART_OVERSAMPLING_16;
  huart1.Init.OneBitSampling = UART_ONE_BIT_SAMPLE_DISABLE;
  huart1.AdvancedInit.AdvFeatureInit = UART_ADVFEATURE_NO_INIT;
  if (HAL_UART_Init(&huart1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN USART1_Init 2 */

  /* USER CODE END USART1_Init 2 */

}

/**
  * Enable DMA controller clock
  */
static void MX_DMA_Init(void)
{

  /* DMA controller clock enable */
  __HAL_RCC_DMA1_CLK_ENABLE();

  /* DMA interrupt init */
  /* DMA1_Channel2_3_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(DMA1_Channel2_3_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(DMA1_Channel2_3_IRQn);

}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void MX_GPIO_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(LED_GPIO_Port, LED_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pins : GPInput1_Pin GPInput2_Pin */
  GPIO_InitStruct.Pin = GPInput1_Pin|GPInput2_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_FALLING;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pin : LED_Pin */
  GPIO_InitStruct.Pin = LED_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(LED_GPIO_Port, &GPIO_InitStruct);

  /* EXTI interrupt init*/
  HAL_NVIC_SetPriority(EXTI0_1_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(EXTI0_1_IRQn);

  HAL_NVIC_SetPriority(EXTI2_3_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(EXTI2_3_IRQn);

}

/* USER CODE BEGIN 4 */
void FuelGauge(int barCount){
	int width = 64;
	int height = 32;

	if(lastFuelGaugeCount != barCount){
		lastFuelGaugeCount = barCount;
		if(barCount == 0){
			for(int i = 0; i<width; i++){
				for(int j = 0; j < height; j++){
					ssd1306_DrawPixel(i,j+32, Black);
				}
			}

			for(int i = width/4; i<width/4+20; i++){
				for(int j = 0; j < height; j++){
					ssd1306_DrawPixel(i,j+32, White);
				}
			}
			for(int i = width/4+2; i<width/4+18; i++){
				for(int j = 2; j < height-20; j++){
					ssd1306_DrawPixel(i,j+32, Black);
				}
			}
			for(int i = 0; i < 3; i++){
				ssd1306_DrawPixel(36,50+i, White);
			}
			for(int i = 0; i < 3; i++){
				ssd1306_DrawPixel(37,50+i, White);
			}
			for(int i = 0; i < 3; i++){
				ssd1306_DrawPixel(38,50+i, White);
			}
			for(int i = 0; i < 3; i++){
				ssd1306_DrawPixel(39,50+i, White);
			}
			for(int i = 0; i < 3; i++){
				ssd1306_DrawPixel(40,51+i, White);
			}
			for(int i = 0; i < 6; i++){
				ssd1306_DrawPixel(41,52+i, White);
			}
			for(int i = 0; i < 6; i++){
				ssd1306_DrawPixel(42,53+i, White);
			}
			for(int i = 0; i < 3; i++){
				ssd1306_DrawPixel(43,56+i, White);
			}
			for(int i = 0; i < 18; i++){
				ssd1306_DrawPixel(44,40+i, White);
			}
			for(int i = 0; i < 16; i++){
				ssd1306_DrawPixel(45,41+i, White);
			}
			for(int i = 0; i < 2; i++){
				ssd1306_DrawPixel(43,39+i, White);
			}
			for(int i = 0; i < 2; i++){
				ssd1306_DrawPixel(42,38+i, White);
			}
			for(int i = 0; i < 2; i++){
				ssd1306_DrawPixel(41,37+i, White);
			}
			for(int i = 0; i < 2; i++){
				ssd1306_DrawPixel(40,36+i, White);
			}
			for(int i = 0; i < 2; i++){
				ssd1306_DrawPixel(39,35+i, White);
			}
			ssd1306_SetCursor(21, 45);
			ssd1306_WriteString("!", Font_11x18, Black);
		}
		if(barCount >= 1){
			for(int i = 0; i<width; i++){
				for(int j = 0; j < height; j++){
					ssd1306_DrawPixel(i,j+32, Black);
				}
			}
			for(int i = 0; i<width/4; i++){
				for(int j = 0; j < height/4; j++){
					ssd1306_DrawPixel(i,63-j, White);
				}
			}
			/*
			for(int i = width/4+1; i<width/2; i++){
				for(int j = 0; j < height/2; j++){
					ssd1306_DrawPixel(i,63-j, Black);
				}
			}*/
		}
		if(barCount >= 2){
			for(int i = width/4+1; i<width/2; i++){
				for(int j = 0; j < height/2; j++){
					ssd1306_DrawPixel(i,63-j, White);
				}
			}
			for(int i = width/2+1; i<width-width/4; i++){
				for(int j = 0; j < height-height/4; j++){
					ssd1306_DrawPixel(i,63-j, Black);
				}
			}
		}
		if(barCount >= 3){
			for(int i = width/2+1; i<width-width/4; i++){
				for(int j = 0; j < height-height/4; j++){
					ssd1306_DrawPixel(i,63-j, White);
				}
			}
			for(int i = width-width/4+1; i<width; i++){
				for(int j = 0; j < height; j++){
					ssd1306_DrawPixel(i,63-j, Black);
				}
			}
		}
		if(barCount >= 4){
			for(int i = width-width/4+1; i<width; i++){
				for(int j = 0; j < height; j++){
					ssd1306_DrawPixel(i,63-j, White);
				}
			}
		}
		ssd1306_UpdateScreen(&hi2c1);
	}
}

void MultiChannelADC(ADC_HandleTypeDef *hadc, uint32_t *outbuffer, uint8_t n_of_channels){
    HAL_ADC_Start(hadc);
    for(uint8_t i = 0; i<n_of_channels; i++){
        HAL_ADC_PollForConversion (hadc, 100);
        outbuffer[i] = HAL_ADC_GetValue(hadc);
    HAL_ADC_Stop(hadc);
    }
}
/*##############################################CALLBACKS##############################################*/

void HAL_TIM_IC_CaptureCallback(TIM_HandleTypeDef *htim){
	//RPM - pulse = 100ns
	if(htim -> Instance == TIM3){
		uint32_t rpmTime;
		uint32_t RpmPulseTime = TIM3->CNT;

		resetTimer = 0;

		if(RpmPulseTime < lastRpmPulseTimCnt){
			rpmTime = (65535 + RpmPulseTime) - lastRpmPulseTimCnt;
		}
		else{
			rpmTime = RpmPulseTime - lastRpmPulseTimCnt;
		}
		rpmSamples[rpmSamplesIndex] = rpmTime;
		rpmSamplesIndex++;

		if(rpmSamplesIndex == 60){
			rpmSamplesIndex = 0;

			for(int i = 0; i < (sizeof(rpmSamples)/sizeof(rpmSamples[0])); i++){
				avgRpmTime += rpmSamples[i];
			}
			avgRpmTime /= (sizeof(rpmSamples)/sizeof(rpmSamples[0]));
			rpm = ((float)TIM3->ARR/(float)avgRpmTime)*60;
			char msg[4];
			sprintf(msg, "%d", rpm);

			for(int i = 0; i<104; i++){
				for(int j = 0; j < 27; j++){
					ssd1306_DrawPixel(i,j, Black);
				}
			}

			ssd1306_SetCursor(0, 0);
			ssd1306_WriteString(msg, Font_16x26, White);
			ssd1306_UpdateScreen(&hi2c1);
		}
	}
	//SPEED - pulse = 100ns
	if(htim -> Instance == TIM14){
		uint32_t spdTime;
		uint32_t SpdPulseTime = TIM14->CNT;

		resetTimer = 0;
		trip += TIRE_CIRC_M/1000;
		all_kms += TIRE_CIRC_M/1000;

		if(SpdPulseTime < lastSpdPulseTimCnt){
			spdTime = (65535 + SpdPulseTime) - lastSpdPulseTimCnt;
		}
		else{
			spdTime = SpdPulseTime - lastSpdPulseTimCnt;
		}
		spdSamples[spdSamplesIndex] = spdTime;
		spdSamplesIndex++;

		if(spdSamplesIndex == 60){
			spdSamplesIndex = 0;

			for(int i = 0; i < (sizeof(spdSamples)/sizeof(spdSamples[0])); i++){
				avgSpdTime += spdSamples[i];
			}
			avgSpdTime /= (sizeof(spdSamples)/sizeof(spdSamples[0]));
			speed = (TIRE_CIRC_M/1000)*(float)(((float)TIM14->ARR/(float)spdTime)*3600);
			char msg[4];
			sprintf(msg, "%d", rpm);

			for(int i = 104; i<128; i++){
				for(int j = 0; j < 27; j++){
					ssd1306_DrawPixel(i,j, Black);
				}
			}

			ssd1306_SetCursor(78, 0);
			ssd1306_WriteString(msg, Font_16x26, White);
			ssd1306_UpdateScreen(&hi2c1);
		}
	}
}

void HAL_TIM_PeriodElapsedCallback(TIM_HandleTypeDef *htim){
	if(htim -> Instance == TIM6){
		char voltage[4];
		char trp[6];
		char km[6];

		resetTimer++;
		sprintf(voltage, "%.1f", battVoltage);
		sprintf(trp, "%.1f", trip);
		sprintf(km, "%.1f", all_kms);

		for(int i = 70; i<128; i++){
			for(int j = 27; j < 63; j++){
				ssd1306_DrawPixel(i,j, Black);
			}
		}
		ssd1306_SetCursor(70, 27);
		ssd1306_WriteString("U: ", Font_7x10, White);
		ssd1306_WriteString(voltage, Font_7x10, White);
		ssd1306_SetCursor(70+7*7, 27);
		ssd1306_WriteString("V", Font_7x10, White);
		ssd1306_SetCursor(70, 39);
		ssd1306_WriteString(km, Font_7x10, White);
		ssd1306_SetCursor(70, 51);
		ssd1306_WriteString(trp, Font_7x10, White);
		ssd1306_UpdateScreen(&hi2c1);
	}
}

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */

  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
